@@ -1,8 +1,7 @@
 import numpy as np
-from time import sleep, time
 
 from csr import PitayaCSR, make_filter
-from utils import start_nginx, stop_nginx, twos_complement
+from utils import twos_complement
 from linien.config import DEFAULT_RAMP_SPEED
 from linien.common import convert_channel_mixing_value, \
     LOW_PASS_FILTER, HIGH_PASS_FILTER, ANALOG_OUT0, MHz
@@ -31,6 +30,13 @@ class Registers:
                 self.acquisition.lock_status_changed(v)
 
         self.parameters.lock.change(lock_status_changed)
+
+        def fetch_quadratures_changed(v):
+            if self.acquisition is not None:
+                self.acquisition.fetch_quadratures_changed(v)
+
+        self.parameters.fetch_quadratures.change(fetch_quadratures_changed)
+
         use_ssh = self.host is not None and self.host not in ('localhost', '127.0.0.1')
         self.acquisition = AcquisitionMaster(use_ssh, self.host)
 
@@ -57,33 +63,39 @@ class Registers:
         lock_changed = lock != self.control.exposed_is_locked
         self.control.exposed_is_locked = lock
 
-        sweep_run = 0 if lock else 1
         new = dict(
-            root_sweep_run=sweep_run,
-            root_sweep_step=int(
+            # sweep run is 1 by default. The gateware automatically takes care
+            # of stopping the sweep run after `request_lock` is set by setting
+            # `sweep.clear`
+            logic_sweep_run=1,
+            logic_sweep_step=int(
                 DEFAULT_RAMP_SPEED * params['ramp_amplitude']
                 / (2 ** params['ramp_speed'])
             ),
-            root_sweep_min=-1 * _max(params['ramp_amplitude'] * 8191),
-            root_sweep_max=_max(params['ramp_amplitude'] * 8191),
+            logic_sweep_min=-1 * _max(params['ramp_amplitude'] * 8191),
+            logic_sweep_max=_max(params['ramp_amplitude'] * 8191),
 
-            root_mod_freq=params['modulation_frequency'],
-            root_mod_amp=params['modulation_amplitude'],
+            logic_mod_freq=params['modulation_frequency'],
+            logic_mod_amp=params['modulation_amplitude'] if params['modulation_frequency'] > 0 else 0,
 
-            root_dual_channel=int(params['dual_channel']),
-            root_chain_a_factor=factor_a,
-            root_chain_b_factor=factor_b,
-            root_chain_a_offset=twos_complement(int(params['offset_a']), 14),
-            root_chain_b_offset=twos_complement(int(params['offset_b']), 14),
-            root_out_offset=int(params['center'] * 8191),
-            root_combined_offset=twos_complement(params['combined_offset'], 14),
+            logic_dual_channel=int(params['dual_channel']),
+            logic_chain_a_factor=factor_a,
+            logic_chain_b_factor=factor_b,
+            logic_chain_a_offset=twos_complement(int(params['offset_a']), 14),
+            logic_chain_b_offset=twos_complement(int(params['offset_b']), 14),
+            logic_out_offset=int(params['center'] * 8191),
+            logic_combined_offset=twos_complement(params['combined_offset'], 14),
 
-            root_control_channel=params['control_channel'],
-            root_mod_channel=params['mod_channel'],
-            root_sweep_channel=params['sweep_channel'],
+            logic_control_channel=params['control_channel'],
+            logic_mod_channel=params['mod_channel'],
+            logic_sweep_channel=params['sweep_channel'],
 
             slow_pid_reset=not params['pid_on_slow_enabled'],
 
+            logic_analog_out_1=params['analog_out_1'],
+            logic_analog_out_2=params['analog_out_2'],
+            logic_analog_out_3=params['analog_out_3'],
+
             # channel A
             fast_a_demod_delay=phase_to_delay(params['demodulation_phase_a']),
             fast_a_demod_multiplier=params['demodulation_multiplier_a'],
@@ -91,9 +103,6 @@ class Registers:
             fast_a_y_tap=2,
             fast_a_dy_sel=self.csr.signal('zero'),
 
-            fast_a_y_hold_en=self.csr.states(),
-            fast_a_y_clear_en=self.csr.states(),
-            fast_a_rx_sel=self.csr.signal('zero'),
             fast_a_invert=int(params['invert_a']),
 
             # channel B
@@ -103,37 +112,38 @@ class Registers:
             fast_b_y_tap=1,
             fast_b_dy_sel=self.csr.signal('zero'),
 
-            fast_b_y_hold_en=self.csr.states(),
-            fast_b_y_clear_en=self.csr.states(),
-            fast_b_rx_sel=self.csr.signal('zero'),
             fast_b_invert=int(params['invert_b']),
 
             # trigger on ramp
             scopegen_external_trigger=1,
 
-            gpio_p_oes=0,
-            gpio_n_oes=0b1,
+            gpio_p_oes=0b11111111,
+            gpio_n_oes=0b11111111,
 
-            gpio_p_outs=0,
-            gpio_n_outs=0b1 if params['lock'] else 0b0,
+            gpio_p_outs=params['gpio_p_out'],
+            gpio_n_outs=params['gpio_n_out'],
 
             gpio_n_do0_en=self.csr.signal('zero'),
             gpio_n_do1_en=self.csr.signal('zero'),
 
-            root_slow_decimation=16,
+            logic_slow_decimation=16,
         )
 
         if lock:
             # display combined error signal and control signal
             new.update({
-                'scopegen_adc_a_sel': self.csr.signal('root_combined_error_signal'),
-                'scopegen_adc_b_sel': self.csr.signal('root_control_signal')
+                'scopegen_adc_a_sel': self.csr.signal('logic_combined_error_signal'),
+                'scopegen_adc_a_q_sel': self.csr.signal('zero'),
+                'scopegen_adc_b_sel': self.csr.signal('logic_control_signal'),
+                'scopegen_adc_b_q_sel': self.csr.signal('zero')
             })
         else:
             # display both demodulated error signals
             new.update({
-                'scopegen_adc_a_sel': self.csr.signal("fast_a_y"),
-                'scopegen_adc_b_sel': self.csr.signal("fast_b_y")
+                'scopegen_adc_a_sel': self.csr.signal("fast_a_out_i"),
+                'scopegen_adc_a_q_sel': self.csr.signal("fast_a_out_q"),
+                'scopegen_adc_b_sel': self.csr.signal("fast_b_out_i"),
+                'scopegen_adc_b_q_sel': self.csr.signal("fast_b_out_q"),
             })
 
 
@@ -157,12 +167,12 @@ class Registers:
         for k, v in new.items():
             self.set(k, int(v))
 
-        if sweep_run and sweep_changed:
+        if not lock and sweep_changed:
             # reset sweep for a short time if the scan range was changed
             # this is needed because otherwise it may take too long before
             # the new scan range is reached --> no scope trigger is sent
-            self.set('root_sweep_run', 0)
-            self.set('root_sweep_run', 1)
+            self.set('logic_sweep_run', 0)
+            self.set('logic_sweep_run', 1)
 
         kp = params['p']
         ki = params['i']
@@ -189,49 +199,52 @@ class Registers:
 
         for chain in ('a', 'b'):
             automatic = params['filter_automatic_%s' % chain]
+            # iir_idx means iir_c or iir_d
             for iir_idx in range(2):
-                iir_name = 'fast_%s_iir_%s' % (chain, ('c', 'd')[iir_idx])
+                # iir_sub_idx means in-phase signal or quadrature signal
+                for iir_sub_idx in range(2):
+                    iir_name = 'fast_%s_iir_%s_%d' % (chain, ('c', 'd')[iir_idx], iir_sub_idx+1)
 
-                if automatic:
-                    filter_enabled = True
-                    filter_type = LOW_PASS_FILTER
-                    filter_frequency = params['modulation_frequency'] / MHz * 1e6 / 2
+                    if automatic:
+                        filter_enabled = True
+                        filter_type = LOW_PASS_FILTER
+                        filter_frequency = params['modulation_frequency'] / MHz * 1e6 / 2
 
-                    # if the filter frequency is too low (< 10Hz), the IIR doesn't
-                    # work properly anymore. In that case, don't filter.
-                    # This is also helpful if the raw (not demodulated) signal
-                    # should be displayed which can be achieved by setting
-                    # modulation frequency to 0.
-                    if filter_frequency < 10:
-                        filter_enabled = False
-                else:
-                    filter_enabled = params['filter_%d_enabled_%s' % (iir_idx + 1, chain)]
-                    filter_type = params['filter_%d_type_%s' % (iir_idx + 1, chain)]
-                    filter_frequency = params['filter_%d_frequency_%s' % (iir_idx + 1, chain)]
-
-                base_freq = 125e6
-
-                if not filter_enabled:
-                    self.set_iir(iir_name, *make_filter('P', k=1))
-                else:
-                    if filter_type == LOW_PASS_FILTER:
-                        self.set_iir(iir_name, *make_filter(
-                            'LP', f=filter_frequency / base_freq, k=1
-                        ))
-                    elif filter_type == HIGH_PASS_FILTER:
-                        self.set_iir(iir_name, *make_filter(
-                            'HP', f=filter_frequency / base_freq, k=1
-                        ))
+                        # if the filter frequency is too low (< 10Hz), the IIR doesn't
+                        # work properly anymore. In that case, don't filter.
+                        # This is also helpful if the raw (not demodulated) signal
+                        # should be displayed which can be achieved by setting
+                        # modulation frequency to 0.
+                        if filter_frequency < 10:
+                            filter_enabled = False
                     else:
-                        raise Exception('unknown filter %s for %s' % (filter_type, iir_name))
+                        filter_enabled = params['filter_%d_enabled_%s' % (iir_idx + 1, chain)]
+                        filter_type = params['filter_%d_type_%s' % (iir_idx + 1, chain)]
+                        filter_frequency = params['filter_%d_frequency_%s' % (iir_idx + 1, chain)]
+
+                    base_freq = 125e6
+
+                    if not filter_enabled:
+                        self.set_iir(iir_name, *make_filter('P', k=1))
+                    else:
+                        if filter_type == LOW_PASS_FILTER:
+                            self.set_iir(iir_name, *make_filter(
+                                'LP', f=filter_frequency / base_freq, k=1
+                            ))
+                        elif filter_type == HIGH_PASS_FILTER:
+                            self.set_iir(iir_name, *make_filter(
+                                'HP', f=filter_frequency / base_freq, k=1
+                            ))
+                        else:
+                            raise Exception('unknown filter %s for %s' % (filter_type, iir_name))
 
         if lock_changed:
             if lock:
                 # set PI parameters
-                self.set_pid(kp, ki, kd, slope, reset=0)
+                self.set_pid(kp, ki, kd, slope, reset=0, request_lock=1)
                 self.set_slow_pid(slow_strength, slow_slope, reset=0)
             else:
-                self.set_pid(0, 0, 0, slope, reset=1)
+                self.set_pid(0, 0, 0, slope, reset=1, request_lock=0)
                 self.set_slow_pid(0, slow_slope, reset=1)
         else:
             if lock:
@@ -239,14 +252,17 @@ class Registers:
                 self.set_pid(kp, ki, kd, slope)
                 self.set_slow_pid(slow_strength, slow_slope)
 
-    def set_pid(self, p, i, d, slope, reset=None):
+    def set_pid(self, p, i, d, slope, reset=None, request_lock=None):
+        if request_lock is not None:
+            self.set('logic_request_lock', request_lock)
+
         sign = -1 if slope else 1
-        self.set('root_pid_kp', p * sign)
-        self.set('root_pid_ki', i * sign)
-        self.set('root_pid_kd', d * sign)
+        self.set('logic_pid_kp', p * sign)
+        self.set('logic_pid_ki', i * sign)
+        self.set('logic_pid_kd', d * sign)
 
         if reset is not None:
-            self.set('root_pid_reset', reset)
+            self.set('logic_pid_reset', reset)
 
     def set_slow_pid(self, strength, slope, reset=None):
         sign = slope
